'       base10.bas
'       
'       Copyright 2009 Zachary Scott <cthug.zs@gmail.com>
'       
'       This program is free software; you can redistribute it and/or modify
'       it under the terms of the GNU General Public License as published by
'       the Free Software Foundation; either version 2 of the License, or
'       (at your option) any later version.
'       
'       This program is distributed in the hope that it will be useful,
'       but WITHOUT ANY WARRANTY; without even the implied warranty of
'       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'       GNU General Public License for more details.
'       
'       You should have received a copy of the GNU General Public License
'       along with this program; if not, write to the Free Software
'       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
'       MA 02110-1301, USA.

' variables
dim shared as string inString, typeFlag
dim shared as ulongint inBase, outp

'
' usage: Displays program usage
'
sub usage alias "usage" ()
	print "Usage: base10 <invalue> <type>"
	print
	print "Type:"
	print chr(9)+"b     - binary         (base 2)"
	print chr(9)+"q     - quaternary     (base 4)"
	print chr(9)+"o     - octal          (base 8)"
	print chr(9)+"h     - hexadecimal    (base 16)"
	print
	print "Example:"
	print chr(9)+"base10 1001010 b"
	print chr(9)+"base10 59FF h"
end sub

'
' pow: like c stdlib 'pow' except takes an integers; 
' returns x to the power of y
'
function pow alias "pow" (byval x as integer, y as integer) as integer
	dim as integer p=1, n
	for n=1 to y step 1
		p=p*x
	next n
	return p
end function

'
' base: Returns integer value of string, base b 
'
function base alias "base" (byval in as string, b as integer) as ulongint
	dim as integer intval, n, num
	for n=1 to len(in) step 1
		' hex
		if b = 16 then
			if mid(in, n, 1) = "A" or mid(in, n, 1) = "a" then
				num=10
			elseif mid(in, n, 1) = "B" or mid(in, n, 1) = "b" then
				num=11
			elseif mid(in, n, 1) = "C" or mid(in, n, 1) = "c" then
				num=12
			elseif mid(in, n, 1) = "D" or mid(in, n, 1) = "d" then
				num=13
			elseif mid(in, n, 1) = "E" or mid(in, n, 1) = "e" then
				num=14
			elseif mid(in, n, 1) = "F" or mid(in, n, 1) = "f" then
				num=15
			else
				num=(asc(mid(in, n, 1))-48)
			end if 
		else
			num=(asc(mid(in, n, 1))-48)
		end if
		' check for error
		if num > (b-1) or num < 0 then 
			print "ERROR: "+inString+" is not base "; inBase 
			end 1
		end if
		intval+=(num*pow(b, len(in)-n))
	next n
	return intval
end function

' get arguments
inString=command(1)
if len(inString)=0 then
	usage()	
	end
end if
typeFlag=command(2)
if len(typeFlag)=0 then
	usage()	
	end
end if

' select typeflag
select case typeFlag
	case "b"
		inBase=2
	case "q"
		inBase=4
	case "o"
		inBase=8
	case "h"
		inBase=16
	case else
		usage()	
		end
end select

' display output
outp=base(inString, inBase)
print
print outp
print
sleep

' EOF
